from abc import ABC
from typing import Optional, Any, Tuple

from thestage.entities.rented_instance import RentedInstanceEntity
from thestage.services.clients.thestage_api.dtos.selfhosted_instance_response import SelfHostedInstanceDto
from thestage.services.abstract_mapper import AbstractMapper


class SelfHostedMapper(AbstractMapper):

    @staticmethod
    def get_exclude_fields() -> Tuple:
        return 'id', 'username', 'ram_size_gb'

    @staticmethod
    def build_entity(item: SelfHostedInstanceDto) -> Optional[RentedInstanceEntity]:
        if not item:
            return None

        gpus = []
        if item.detected_gpus:
            gpus = [item.type.value for item in item.detected_gpus.gpus]

        if len(gpus) == 0:
            gpus = ['NO_GPU']

        return RentedInstanceEntity(
            id=item.id,
            slug=item.slug,
            title=item.title,
            cpu_type=item.cpu_type,
            cpu_cores=item.cpu_cores,
            gpu_type=', '.join(gpus),
            ram_size_gb=item.ram_size_gb,
            ip_address=item.ip_address,
            username=item.host_username,
            status=item.frontend_status.status_translation if item.frontend_status else None,
            created_at=item.created_at.strftime("%Y-%m-%d %H:%M:%S") if item.created_at else '',
            updated_at=item.updated_at.strftime("%Y-%m-%d %H:%M:%S") if item.updated_at else '',
        )



import os
from pathlib import Path
from typing import Optional

from thestage_core.services.config_provider.config_provider import ConfigProviderCore

from thestage.services.project.dto.project_config import ProjectConfig


class ConfigProvider(ConfigProviderCore):
    def __init__(
            self,
            local_path: str,
    ):
        super(ConfigProvider, self).__init__(
            local_path=local_path,
        )

    def save_project_config(self, project_config: ProjectConfig):
        self.__create_empty_project_config_if_missing()
        # self.read_project_config()
        project_config_path = self.__get_project_config_path(with_file=True)
        self._save_config_file(data=project_config.model_dump(), file_path=project_config_path)

    def save_project_deploy_ssh_key(self, slug: str, deploy_ssh_key: str) -> str:
        deploy_key_dirpath = self._global_config_path.joinpath('project_deploy_keys')
        self._file_system_service.create_if_not_exists(deploy_key_dirpath)

        deploy_key_filepath = deploy_key_dirpath.joinpath('project_deploy_key_' + slug)
        self._file_system_service.create_if_not_exists_file(deploy_key_filepath)

        text_file = open(deploy_key_filepath, "w")
        text_file.write(deploy_ssh_key)
        text_file.close()
        os.chmod(deploy_key_filepath, 0o600)

        return str(deploy_key_filepath)

    def read_project_config(self) -> Optional[ProjectConfig]:
        project_data_dirpath = self.__get_project_config_path()
        if not project_data_dirpath.exists():
            return None
            # self._file_system_service.create_if_not_exists(project_data_dirpath)

        project_data_filepath = self.__get_project_config_path(with_file=True)
        if not project_data_filepath.exists():
            return None

        config_data = self._read_config_file(project_data_filepath) if project_data_filepath and project_data_filepath.exists() else {}
        return ProjectConfig.model_validate(config_data)


    def __create_empty_project_config_if_missing(self):
        project_data_dirpath = self.__get_project_config_path()
        if not project_data_dirpath.exists():
            self._file_system_service.create_if_not_exists(project_data_dirpath)

        project_data_filepath = self.__get_project_config_path(with_file=True)
        if not project_data_filepath.exists():
            self._file_system_service.create_if_not_exists_file(project_data_filepath)


    def __get_project_config_path(self, with_file: bool = False) -> Path:
        if with_file:
            return self._local_config_path.joinpath('project.json')
        else:
            return self._local_config_path


import datetime
import os
from pathlib import Path
from typing import Optional, List, Tuple, Dict

import git
import typer
from git import Remote, Repo, GitCommandError, Commit, Head, RemoteReference, Git
from thestage_core.services.filesystem_service import FileSystemServiceCore

from thestage.exceptions.git_access_exception import GitAccessException


class GitLocalClient:
    __base_name_remote: str = 'origin'
    __base_name_local: str = 'main'
    __git_ignore_thestage_line: str = '/.thestage/'

    __special_main_branches = ['main', 'master']

    __base_git_url: str = 'https://github.com/'

    def __init__(
            self,
            file_system_service: FileSystemServiceCore,
    ):
        self.__file_system_service = file_system_service

    def __get_repo(self, path: str) -> Repo:
        return git.Repo(path)

    def is_present_local_git(self, path: str) -> bool:
        git_path = self.__file_system_service.get_path(path)
        if not git_path.exists():
            return False

        git_path = git_path.joinpath('.git')
        if not git_path.exists():
            return False

        result = git.repo.base.is_git_dir(git_path)
        return result

    def get_remote(self, path: str) -> Optional[List[Remote]]:
        is_git_repo = self.is_present_local_git(path=path)
        if is_git_repo:
            repo = git.Repo(path)
            remotes: Optional[List[Remote]] = list(repo.remotes) if repo.remotes else []
            return remotes
        return None

    def has_remote(self, path: str) -> bool:
        remotes: Optional[List[Remote]] = self.get_remote(path)
        return True if remotes is not None and len(remotes) > 0 else False

    def has_changes(self, path: str) -> bool:
        repo = self.__get_repo(path=path)
        return repo.is_dirty()

    def init_repository(
            self,
            path: str,
    ) -> Optional[Repo]:

        repo = git.Repo.init(path)
        if repo:
            # default git name master, rename to main - sync wih github
            repo.git.branch("-M", self.__base_name_local)
        return repo

    def add_remote_to_repo(
            self,
            path: str,
            remote_url: str,
            remote_name: str,
    ) -> bool:
        repo = self.__get_repo(path=path)
        remotes: List[Remote] = repo.remotes
        not_present = True
        if remotes:
            item = list(filter(lambda x: x.name == remote_name, remotes))
            if len(item) > 0:
                not_present = False

        if not_present:
            remote: Remote = repo.create_remote(
                name=self.__base_name_remote,
                url=remote_url,
            )
            if remote:
                return True
            else:
                return False
        else:
            return True

    def git_fetch(self, path: str, deploy_key_path: str):
        repo = self.__get_repo(path=path)
        git_ssh_cmd = 'ssh -F /dev/null -o StrictHostKeyChecking=no -o IdentitiesOnly=yes -i %s' % deploy_key_path

        with repo.git.custom_environment(GIT_SSH_COMMAND=git_ssh_cmd):
            remote: Remote = repo.remote(self.__base_name_remote)
            if remote:
                try:
                    remote.fetch()
                except GitCommandError as base_ex:
                    msg = base_ex.stderr
                    if msg and 'fatal: Could not read from remote repository' in msg:
                        raise GitAccessException(
                            message='You dont have access to repository, or repository not found.',
                            url=self.build_http_repo_url(git_path=remote.url),
                            dop_message=msg,
                        )
                    else:
                        raise base_ex

    def git_pull(self, path: str, deploy_key_path: str, branch: Optional[str] = None):
        repo = self.__get_repo(path=path)
        git_ssh_cmd = 'ssh -F /dev/null -o StrictHostKeyChecking=no -o IdentitiesOnly=yes -i %s' % deploy_key_path

        with repo.git.custom_environment(GIT_SSH_COMMAND=git_ssh_cmd):
            local_branch = self.__base_name_local
            if branch:
                if self.__base_name_remote in branch:
                    local_branch = branch.replace(f"{self.__base_name_remote}/", '').strip()
                else:
                    local_branch = branch

            repo.git.pull(self.__base_name_remote, local_branch)

    def find_main_branch_name(self, path: str) -> Optional[str]:
        repo = self.__get_repo(path=path)
        if repo:
            for ref in repo.git.branch('-a').split('\n'):
                for main_branch in self.__special_main_branches:
                    if main_branch in ref.split('/'):
                        if 'remotes/' in ref:
                            return ref.replace('remotes/', '').strip()
                        else:
                            return ref.strip()
        return None

    def get_active_branch_name(self, path: str) -> Optional[str]:
        repo = self.__get_repo(path=path)
        if repo:
            return repo.active_branch.name
        return None

    def git_checkout(self, path: str, branch: Optional[str] = None):
        repo = self.__get_repo(path=path)
        if repo:
            branch_name = repo.active_branch.name if repo.active_branch else self.__base_name_local
            if branch:
                branch_name = branch

            repo.git.checkout('-b', branch_name.strip())

    def add_new_branch(self, path: str, new_branch_name: Optional[str] = None,) -> Optional[Head]:
        # TODO: check and changes on future, fucking branches
        repo = self.__get_repo(path=path)
        if repo:
            branch = self.find_main_branch_name(path=path)
            if branch:
                return branch
            else:
                branch_name = new_branch_name if new_branch_name else self.__base_name_local

            origin = repo.remote(self.__base_name_remote)
            if origin:
                repo.create_head(self.__base_name_local, origin.refs.main)  # create local branch "master" from remote "master"
                repo.heads.main.set_tracking_branch(origin.refs.main)  # set local "master" to track remote "master
                repo.heads.master.checkout()

            #if new_branch_name:
                branch = repo.create_head(branch_name)
            #else:
                # if we want added special branch - main - first branch
                #origin = repo.remote(self.__base_name_remote)
            #    repo.head.reference = repo.create_head(branch_name)
                # Create new remote ref and set it to track.
            #    rem_ref = RemoteReference(repo, f"refs/remotes/{self.__base_name_remote}/{branch_name}")
            #    repo.head.reference.set_tracking_branch(rem_ref)

            return branch
        return None

    def build_http_repo_url(self, git_path: str) -> str:
        start_path_pos = git_path.find(":")
        pre_url = git_path[start_path_pos+1:]
        url = pre_url.replace('.git', '')
        return self.__base_git_url + url

    def clone(self, url: str, path: str, deploy_key_path: str) -> Optional[Repo]:
        try:
            git_ssh_cmd = 'ssh -F /dev/null -o StrictHostKeyChecking=no -o IdentitiesOnly=yes -i %s' % deploy_key_path
            return Repo.clone_from(url=url, to_path=path, env={"GIT_SSH_COMMAND": git_ssh_cmd})
        except GitCommandError as base_ex:
            msg = base_ex.stderr
            if msg and 'Repository not found' in msg and 'correct access rights' in msg:
                raise GitAccessException(
                    message='You dont have access to repository, or repository not found.',
                    url=self.build_http_repo_url(git_path=url),
                    dop_message=msg,
                )
            else:
                raise base_ex

    def commit_local_changes(
            self,
            path: str,
            name: Optional[str] = None,
    ) -> Optional[str]:
        repo = self.__get_repo(path=path)
        commit_name = name if name else f"Auto commit {str(datetime.datetime.now().date())}"
        commit = repo.git.commit('--all', '--allow-empty', '-m', commit_name, )
        return commit

    def push_changes(
            self,
            path: str,
            deploy_key_path: str
    ):
        repo = self.__get_repo(path=path)
        git_ssh_cmd = 'ssh -F /dev/null -o StrictHostKeyChecking=no -o IdentitiesOnly=yes -i %s' % deploy_key_path

        with repo.git.custom_environment(GIT_SSH_COMMAND=git_ssh_cmd):
            origin = repo.remote(self.__base_name_remote)
            if origin:
                return repo.git.push(origin.name, repo.active_branch.name)

    def get_latest_commit(self, path: str, ) -> Optional[Commit]:
        repo = self.__get_repo(path=path)
        if repo:
            return repo.head.commit
        else:
            return None

    def _get_gitignore_path(self, path: str) -> Path:
        git_path = self.__file_system_service.get_path(path)
        return git_path.joinpath('.gitignore')

    def git_add_by_path(self, repo_path: str, file_path: str):
        repo = self.__get_repo(path=repo_path)
        if repo:
            repo.index.add(items=[file_path])

    def git_add_all(self, repo_path: str):
        repo = self.__get_repo(path=repo_path)
        if repo:
            repo.git.add(all=True)

    def init_gitignore(self, path: str):
        gitignore_path = self._get_gitignore_path(path=path)
        if not gitignore_path.exists():
            self.__file_system_service.create_if_not_exists_file(gitignore_path)
            self.git_add_by_path(repo_path=path, file_path=str(gitignore_path))

        is_present_tsr = self.__file_system_service.find_in_text_file(file=str(gitignore_path),
                                                                      find=self.__git_ignore_thestage_line)
        if not is_present_tsr:
            self.__file_system_service.add_line_to_text_file(file=str(gitignore_path),
                                                             new_line=self.__git_ignore_thestage_line)



from pydantic import Field, ConfigDict, BaseModel


class ProjectGetDeploySshKeyRequest(BaseModel):
    model_config = ConfigDict(use_enum_values=True)

    projectSlug: str = Field(None, alias='projectSlug')



from typing import Optional, Tuple

from thestage.services.clients.thestage_api.dtos.container_response import DockerContainerDto
from thestage.entities.container import DockerContainerEntity
from thestage.services.abstract_mapper import AbstractMapper


class ContainerMapper(AbstractMapper):

    @staticmethod
    def get_exclude_fields() -> Tuple:
        return ('id', 'system_name')

    @staticmethod
    def build_entity(item: DockerContainerDto) -> Optional[DockerContainerEntity]:
        if not item:
            return None

        return DockerContainerEntity(
            id=item.id,
            status=item.frontend_status.status_translation if item.frontend_status else None,
            slug=item.slug,
            title=item.title,
            instance_rented_slug=item.instance_rented.slug if item.instance_rented else '',
            selfhosted_rented_slug=item.selfhosted_instance.slug if item.selfhosted_instance else '',
            system_name=item.system_name,
            docker_image=item.docker_image,
        )



from typing import Optional

from pydantic import Field, ConfigDict, BaseModel


class ProjectRunTaskRequest(BaseModel):
    model_config = ConfigDict(use_enum_values=True)

    projectSlug: str = Field(None, alias='projectSlug')
    instanceRentedSlug: Optional[str] = Field(None, alias='instanceRentedSlug')
    selfhostedInstanceSlug: Optional[str] = Field(None, alias='selfhostedInstanceSlug')
    dockerContainerSlug: Optional[str] = Field(None, alias='dockerContainerSlug')
    commitHash: Optional[str] = Field(None, alias='commitHash')
    runCommand: str = Field(None, alias='runCommand')
    taskTitle: Optional[str] = Field(None, alias='taskTitle')
    taskDescription: Optional[str] = Field(None, alias='taskDescription')



from abc import ABC
from typing import Any, Optional, Tuple


class AbstractMapper(ABC):

    @staticmethod
    def get_exclude_fields() -> Tuple:
        return ()

    @staticmethod
    def build_entity(item: Any) -> Optional[Any]:
        raise NotImplementedError()



from typing import Optional

from thestage.entities.project_task import ProjectTaskEntity
from thestage.services.clients.thestage_api.dtos.task_response import TaskDto


class ProjectTaskMapper:

    @staticmethod
    def build_task_entity(item: TaskDto) -> Optional[ProjectTaskEntity]:
        if not item:
            return None

        return ProjectTaskEntity(
            id=item.id,
            title=item.title,
            status=item.frontend_status.status_translation,
            docker_container_id=item.docker_container_id,
            started_at=item.started_at,
            finished_at=item.finished_at,
        )



from enum import Enum
from typing import List


# deprecated later
class RentedStatusEnumDto(str, Enum):
    NEW: str = 'NEW'
    AWAITING_RENT: str = 'AWAITING_RENT'
    RENTING: str = 'RENTING'
    RENTED: str = 'RENTED'
    RENTING_FAILED: str = 'RENTING_FAILED'
    AWAITING_TERMINATE: str = 'AWAITING_TERMINATE'
    TERMINATING: str = 'TERMINATING'
    TERMINATED: str = 'TERMINATED'
    TERMINATING_FAILED: str = 'TERMINATING_FAILED'
    UNKNOWN: str = 'UNKNOWN'
    ALL: str = 'ALL'

    @staticmethod
    def find_special_status(statuses: List['RentedStatusEnumDto']) -> bool:
        q = list(filter(lambda x: True if x == RentedStatusEnumDto.ALL else False, statuses))
        return True if q else False


class RentedBusinessStatusEnumDto(str, Enum):

    IN_QUEUE: str = 'IN_QUEUE'
    CREATING: str = 'CREATING'
    ONLINE: str = 'ONLINE'
    TERMINATING: str = 'TERMINATING'
    STOPPED: str = 'STOPPED'
    STOPPING: str = 'STOPPING'
    STARTING: str = 'STARTING'
    REBOOTING: str = 'REBOOTING'
    DELETED: str = 'DELETED'
    RENTAL_ERROR: str = 'RENTAL_ERROR'
    UNKNOWN: str = 'UNKNOWN'
    ALL: str = 'ALL'

    @staticmethod
    def find_special_status(statuses: List['RentedBusinessStatusEnumDto']) -> bool:
        q = list(filter(lambda x: True if x == RentedBusinessStatusEnumDto.ALL else False, statuses))
        return True if q else False


class RentedFrontendStatusEnumDto(str, Enum):

    terminated: str = 'terminated'
    online: str = 'online'
    rebooting: str = 'rebooting'
    terminating: str = 'terminating'
    stopped: str = 'stopped'
    stopping: str = 'stopping'
    starting: str = 'starting'
    rental_error: str = 'rental_error'
    creating: str = 'creating'
    unknown: str = 'unknown'
    all: str = 'all'

    @staticmethod
    def find_special_status(statuses: List['RentedFrontendStatusEnumDto']) -> bool:
        q = list(filter(lambda x: True if x == RentedFrontendStatusEnumDto.all else False, statuses))
        return True if q else False



from datetime import datetime
from typing import Optional, List, Dict

from pydantic import Field, BaseModel, ConfigDict

from thestage.services.clients.thestage_api.dtos.enums.daemon_status import DaemonStatusEnumDto
from thestage.services.clients.thestage_api.dtos.frontend_status import FrontendStatusDto
from thestage.services.clients.thestage_api.dtos.instance_detected_gpus import InstanceDetectedGpusDto
from thestage.services.clients.thestage_api.dtos.enums.instance_type import InstanceTypeEnumDto
from thestage.services.clients.thestage_api.dtos.base_response import TheStageBaseResponse, TheStageBasePaginatedResponse
from thestage.services.clients.thestage_api.dtos.pagination_data import PaginationData
from thestage.services.clients.thestage_api.dtos.cloud_provider_region import CloudProviderRegionDto
from thestage.services.clients.thestage_api.dtos.enums.disk_type import DiskTypeEnumDto
from thestage.services.clients.thestage_api.dtos.enums.gpu_name import GpuNameEnumDto
from thestage.services.clients.thestage_api.dtos.enums.power_status import PowerStatusEnumDto
from thestage.services.clients.thestage_api.dtos.enums.rented_status import RentedStatusEnumDto, \
    RentedBusinessStatusEnumDto
from thestage.services.clients.thestage_api.dtos.price_definition import PriceDefinitionDto
from thestage.services.clients.thestage_api.dtos.enums.cpu_type import CpuTypeEnumDto


class InstanceRentedDto(BaseModel):
    model_config = ConfigDict(use_enum_values=True)

    id: Optional[int] = Field(None, alias='id')
    display_price_per_hour: Optional[PriceDefinitionDto] = Field(None, alias='displayPricePerHour')
    total_money_spent: Optional[List[PriceDefinitionDto]] = Field(default_factory=list, alias='totalMoneySpent')
    client_id: Optional[int] = Field(None, alias='clientId')
    instance_type: Optional[InstanceTypeEnumDto] = Field(InstanceTypeEnumDto.UNKNOWN, alias='instanceType', validate_default=True)
    provider_id: Optional[str] = Field(None, alias='providerId')
    cloud_provider_region_id: Optional[int] = Field(None, alias='cloudProviderRegionId')
    cloud_provider_region: Optional[CloudProviderRegionDto] = Field(None, alias='cloudProviderRegion')
    cpu_type: Optional[CpuTypeEnumDto] = Field(CpuTypeEnumDto.UNKNOWN, alias='cpuType')
    cpu_name: Optional[str] = Field(None, alias='cpuName')
    cpu_cores: Optional[int] = Field(None, alias='cpuCores')
    gpu_type: Optional[GpuNameEnumDto] = Field(Optional[GpuNameEnumDto.UNKNOWN], alias='gpuType')
    gpu_name: Optional[str] = Field(None, alias='gpuName')
    gpu_memory_gb: Optional[int] = Field(None, alias='gpuMemoryGb')
    gpu_count: Optional[int] = Field(None, alias='gpuCount')
    disk_type: Optional[DiskTypeEnumDto] = Field(DiskTypeEnumDto.UNKNOWN, alias='diskType')
    disk_size_gb: Optional[int] = Field(None, alias='diskSizeGb')
    ram_size_gb: Optional[int] = Field(None, alias='ramSizeGb')
    average_renting_time_minutes: Optional[int] = Field(None, alias='averageRentingTimeMinutes')

    business_status: RentedBusinessStatusEnumDto = Field(RentedBusinessStatusEnumDto.UNKNOWN, alias='businessStatus')
    frontend_status: Optional[FrontendStatusDto] = Field(None, alias='frontendStatus')

    slug: Optional[str] = Field(None, alias='slug')
    title: Optional[str] = Field(None, alias='title')
    power_status: PowerStatusEnumDto = Field(PowerStatusEnumDto.UNKNOWN, alias='powerStatus')
    created_at: Optional[datetime] = Field(None, alias='createdAt')
    updated_at: Optional[datetime] = Field(None, alias='updatedAt')
    ip_address: Optional[str] = Field(None, alias='ipAddress')
    host_username: Optional[str] = Field(None, alias='hostUsername')
    detected_gpus: Optional[InstanceDetectedGpusDto] = Field(None, alias='detectedGpus')
    daemon_status: Optional[DaemonStatusEnumDto] = Field(DaemonStatusEnumDto.UNKNOWN, alias='daemonStatus')


class InstanceRentedListPaginated(TheStageBasePaginatedResponse):
    model_config = ConfigDict(use_enum_values=True)

    entities: List[InstanceRentedDto] = Field(default_factory=list, alias='entities')
    pagination_data: Optional[PaginationData] = Field(None, alias='paginationData')


class InstanceRentedListResponse(TheStageBasePaginatedResponse):
    paginated_list: Optional[InstanceRentedListPaginated] = Field(None, alias='paginatedEntityList')


class InstanceRentedItemResponse(TheStageBasePaginatedResponse):
    model_config = ConfigDict(use_enum_values=True)

    instance_rented: Optional[InstanceRentedDto] = Field(None, alias='instanceRented')


class InstanceRentedBusinessStatusMapperResponse(TheStageBasePaginatedResponse):
    model_config = ConfigDict(use_enum_values=True)

    instance_rented_business_status_map: Dict[str, str] = Field(default=dict, alias='instanceRentedBusinessStatusMap')



from typing import Dict, Optional

import typer
from thestage_core.entities.config_entity import ConfigEntity, MainConfigEntity
from thestage_core.services.validation_service import ValidationServiceCore

from thestage.i18n.translation import __
from thestage.services.config_provider.config_provider import ConfigProvider
from thestage.services.clients.thestage_api.api_client import TheStageApiClient


class ValidationService(ValidationServiceCore):
    _thestage_api_client: TheStageApiClient = None

    def __init__(
            self,
            thestage_api_client: TheStageApiClient,
            config_provider: ConfigProvider,
    ):
        super(ValidationService, self).__init__(
            thestage_api_client=thestage_api_client,
            config_provider=config_provider,
        )

    def check_token(
            self,
            config: ConfigEntity,
            no_dialog: bool = False,
    ):

        is_present = self.is_present_token(config=config)

        token = config.main.auth_token if config and config.main else None
        if not is_present:

            if no_dialog:
                typer.echo(__(
                    'In no dialog mode, you need added token to config or to system env: THESTAGE_AUTH_TOKEN'
                ))
                raise typer.Exit(1)

            new_token: str = typer.prompt(
                text=__('Please sign in via valid TheStage.ai API token'),
                show_choices=False,
                type=str,
                show_default=False,
            )
            token = new_token

        is_valid = self.validate_token(token,)
        if not is_valid:
            typer.echo(__(
                'Please issue the valid API token in your TheStage.ai web app account'
            ))
            raise typer.Exit(1)

        typer.echo(__('Token is valid'))
        if not is_present:

            if config.main:
                config.main.auth_token = token
            else:
                config.main = MainConfigEntity(tsr_auth_token=token)



from typing import Optional, List

from thestage.helpers.logger.app_logger import app_logger
from thestage.services.clients.thestage_api.dtos.enums.selfhosted_status import SelfHostedFrontendStatusEnumDto
from thestage.services.clients.thestage_api.dtos.enums.rented_status import RentedFrontendStatusEnumDto
from thestage.services.instance.mapper.instance_mapper import InstanceMapper
from thestage.services.instance.mapper.selfhosted_mapper import SelfHostedMapper
from thestage.services.instance.instance_service import InstanceService
from thestage.i18n.translation import __
from thestage.controllers.utils_controller import \
    validate_config_and_get_service_factory

import typer


app = typer.Typer(no_args_is_help=True, help=__("Help working with instances"))

rented = typer.Typer(no_args_is_help=True, help=__("Help working with rented instances"))
self_hosted = typer.Typer(no_args_is_help=True, help=__("Help working with self hosted instances"))

app.add_typer(rented, name="rented")
app.add_typer(self_hosted, name="self-hosted")


@rented.command(name="ls", help=__("Show list rented instances"))
def rented_list(
        row: int = typer.Option(
            5,
            '--row',
            '-r',
            help=__("Count row in table"),
            is_eager=False,
        ),
        page: int = typer.Option(
            1,
            '--page',
            '-p',
            help=__("Page number"),
            is_eager=False,
        ),
        statuses: List[RentedFrontendStatusEnumDto] = typer.Option(
            ["online", "creating", "terminating", "rebooting"],
            '--status',
            '-s',
            help=__("Status item (all - show all instances)"),
            is_eager=False,
        ),
        no_dialog: Optional[bool] = typer.Option(
            None,
            "--no-dialog",
            "-nd",
            help=__("Start process with default values, without future dialog"),
            is_eager=False,
        ),
):
    """
        List rented instances
    """
    service_factory = validate_config_and_get_service_factory(no_dialog=no_dialog)
    config = service_factory.get_config_provider().get_full_config()

    headers = [
        #'#',
        #'ID',
        'STATUS',
        'TITLE',
        'UNIQUE ID',
        'CPU TYPE',
        'CPU CORES',
        'GPU TYPE',
        'IP ADDRESS',
        'CREATED AT',
        'UPDATED AT',
    ]

    instance_service: InstanceService = service_factory.get_instance_service()

    typer.echo(__(
        "Start show instance list with statuses: %statuses% (You can show all statuses set up status=all)",
        placeholders={
            'statuses': ', '.join([item.value for item in statuses])
        }))

    real_statuses: List[str] = instance_service.map_rented_statuses(config=config, frontend=statuses)

    instance_service.print(
        func_get_data=instance_service.get_rented_list,
        func_special_params={
            'statuses': real_statuses,
        },
        mapper=InstanceMapper(),
        config=config,
        headers=headers,
        row=row,
        page=page,
        no_dialog=no_dialog,
        show_index="never",
    )

    typer.echo(__("List rented done"))
    raise typer.Exit(0)


@rented.command(name="connect", help=__("Help connect to instance"))
def instance_connect(
        instance_slug: Optional[str] = typer.Option(
            None,
            '--instance-uniqueid',
            '-uid',
            help=__("Instance unique id"),
            is_eager=False,
        ),
        no_dialog: Optional[bool] = typer.Option(
            None,
            "--no-dialog",
            "-nd",
            help=__("Start process with default values, without future dialog"),
            is_eager=False,
        ),
):
    """
        Connect to instance
    """
    app_logger.info(f'Connect to instance')

    if not instance_slug:
        typer.echo(__('Instance unique id is required'))
        raise typer.Exit(1)

    service_factory = validate_config_and_get_service_factory(no_dialog=no_dialog)
    config = service_factory.get_config_provider().get_full_config()

    instance_service: InstanceService = service_factory.get_instance_service()
    instance = instance_service.get_rented_item(config=config, instance_slug=instance_slug)

    if instance:
        instance_service.check_instance_status_to_connect(
            instance=instance,
        )
        instance_service.connect_to_instance(
            ip_address=instance.ip_address,
            username=instance.host_username,
        )
    else:
        typer.echo(__("Not found instance - %instance_item%", {'instance_item': instance_slug}))

    app_logger.info(f'Stop connect to instance')
    raise typer.Exit(0)


@self_hosted.command(name="ls", help=__("Show list self-hosted instances"))
def self_hosted_list(
        row: int = typer.Option(
            5,
            '--row',
            '-r',
            help=__("Count row in table"),
            is_eager=False,
        ),
        page: int = typer.Option(
            1,
            '--page',
            '-p',
            help=__("Page number"),
            is_eager=False,
        ),
        statuses: List[SelfHostedFrontendStatusEnumDto] = typer.Option(
            ["awaiting_setup", "online", "terminated"],
            '--status',
            '-s',
            help=__("Status item (all - show all instances)"),
            is_eager=False,
        ),
        no_dialog: Optional[bool] = typer.Option(
            None,
            "--no-dialog",
            "-nd",
            help=__("Start process with default values, without future dialog"),
            is_eager=False,
        ),
):
    """
        List self hosted instances
    """
    service_factory = validate_config_and_get_service_factory(no_dialog=no_dialog)
    config = service_factory.get_config_provider().get_full_config()

    headers = [
        # '#',
        # 'ID',
        'STATUS',
        'TITLE',
        'UNIQUE ID',
        'CPU TYPE',
        'CPU CORES',
        'GPU TYPE',
        'IP ADDRESS',
        'CREATED AT',
        'UPDATED AT'
    ]

    instance_service: InstanceService = service_factory.get_instance_service()

    typer.echo(__(
        "Start show self-hosted list with statuses: %statuses% (You can show all statuses set up status=all)",
        placeholders={
            'statuses': ', '.join([item.value for item in statuses])
        }))

    real_statuses: List[str] = instance_service.map_selfhosted_statuses(config=config, frontend=statuses)

    instance_service.print(
        func_get_data=instance_service.get_self_hosted_list,
        func_special_params={
            'statuses': real_statuses,
        },
        mapper=SelfHostedMapper(),
        config=config,
        headers=headers,
        row=row,
        page=page,
        no_dialog=no_dialog,
        show_index="never",
    )

    typer.echo(__("List self-hosted done"))
    raise typer.Exit(0)


@self_hosted.command(name="connect", help=__("Help connect to self hosted"))
def self_hosted_connect(
        instance_slug: Optional[str] = typer.Option(
            None,
            '--instance-uniqueid',
            '-uid',
            help=__("Instance unique id"),
            is_eager=False,
        ),
        username: Optional[str] = typer.Option(
            'root',
            '--username',
            '-u',
            help=__("Instance username if use selfhosted"),
            is_eager=False,
        ),
        no_dialog: Optional[bool] = typer.Option(
            None,
            "--no-dialog",
            "-nd",
            help=__("Start process with default values, without future dialog"),
            is_eager=False,
        ),
):
    """
        Connect to self hosted instances
    """
    app_logger.info(f'Connect to self hosted instance')

    if not instance_slug:
        typer.echo(__('Instance unique id is required'))
        raise typer.Exit(1)

    service_factory = validate_config_and_get_service_factory(no_dialog=no_dialog)
    config = service_factory.get_config_provider().get_full_config()

    instance_service: InstanceService = service_factory.get_instance_service()
    # rebuild to rented
    instance = instance_service.get_self_hosted_item(config=config, instance_slug=instance_slug)

    if instance:
        instance_service.check_selfhosted_status_to_connect(
            instance=instance,
        )

        if not username or not instance.host_username:
            typer.echo(__('Not found user connect ti instance'))
            raise typer.Exit(1)

        instance_service.connect_to_instance(
            ip_address=instance.ip_address,
            username=username or instance.host_username or 'root',
        )
    else:
        typer.echo(__("Not found instance - %instance_item%", {'instance_item': instance_slug}))

    app_logger.info(f'Stop connect to instance')
    raise typer.Exit(0)



from typing import Optional, List, Tuple, Dict

import typer
from thestage_core.services.clients.thestage_api.api_client import TheStageApiClientCore

from thestage.config import THESTAGE_API_URL
from thestage.services.clients.thestage_api.dtos.entity_filter_request import EntityFilterRequest
from thestage.services.clients.thestage_api.dtos.project_controller.project_get_deploy_ssh_key_request import \
    ProjectGetDeploySshKeyRequest
from thestage.services.clients.thestage_api.dtos.project_controller.project_get_deploy_ssh_key_response import \
    ProjectGetDeploySshKeyResponse
from thestage.services.clients.thestage_api.dtos.project_controller.project_run_task_request import \
    ProjectRunTaskRequest
from thestage.services.clients.thestage_api.dtos.project_controller.project_run_task_response import \
    ProjectRunTaskResponse
from thestage.services.clients.thestage_api.dtos.project_response import ProjectDto, ProjectViewResponse
from thestage.services.clients.thestage_api.dtos.selfhosted_instance_response import SelfHostedInstanceListResponse, \
    SelfHostedInstanceDto, SelfHostedRentedItemResponse, SelfHostedRentedRentedBusinessStatusMapperResponse
from thestage.services.clients.thestage_api.dtos.container_param_request import DockerContainerActionRequestDto
from thestage.services.clients.thestage_api.dtos.container_response import DockerContainerListResponse, \
    DockerContainerDto, \
    DockerContainerItemResponse, ContainerBusinessStatusMapperResponse
from thestage.entities.enums.order_direction_type import OrderDirectionType
from thestage.services.clients.thestage_api.dtos.task_controller.task_list_for_project_request import \
    TaskListForProjectRequest
from thestage.services.clients.thestage_api.dtos.task_controller.task_list_for_project_response import \
    TaskListForProjectResponse, TaskListForProjectPaging
from thestage.services.clients.thestage_api.dtos.user_profile import UserProfileResponse
from thestage.services.clients.thestage_api.dtos.enums.rented_status import RentedStatusEnumDto, \
    RentedBusinessStatusEnumDto
from thestage.services.clients.thestage_api.dtos.instance_rented_response import InstanceRentedListResponse, \
    InstanceRentedDto, InstanceRentedItemResponse, InstanceRentedBusinessStatusMapperResponse
from thestage.services.clients.thestage_api.dtos.storage_rented_response import StorageRentedListResponse, \
    StorageRentedDto
from thestage.services.clients.thestage_api.dtos.task_response import TaskViewResponse, TaskGetOutputResponse
from thestage.services.clients.thestage_api.dtos.base_response import TheStageBaseResponse, \
    TheStageBasePaginatedResponse


class TheStageApiClient(TheStageApiClientCore):
    def __init__(self, api_url: str, timeout: int = 90):
        super().__init__(timeout=timeout, url=api_url)

    def get_task_info_by_task_id(
            self,
            token: str,
            task_id: int,
    ) -> Optional[TaskViewResponse]:
        data = {
            "taskId": task_id,
        }

        response = self._request(
            method='POST',
            url='/user-api/v1/task/view',
            data=data,
            token=token,
        )

        result = TaskViewResponse.model_validate(response) if response else None
        return result if result and result.is_success else None

    def get_task_list_for_project(
            self,
            token: str,
            project_slug: str,
            page: int = 1,
            limit: int = 10,
    ) -> Optional[TaskListForProjectPaging]:
        request = TaskListForProjectRequest(
            projectSlug=project_slug,
            entityFilterRequest=EntityFilterRequest(
                orderByField="id",
                orderByDirection=OrderDirectionType.DESC,
                page=page,
                limit=limit,
            ),
        )

        response = self._request(
            method='POST',
            url='/user-api/v1/task/list-for-project',
            data=request.model_dump(),
            token=token,
        )

        result = TaskListForProjectResponse.model_validate(response) if response else None
        return result.tasks if result and result.is_success else None

    def get_output_task(
            self,
            token: str,
            task_id: int,
    ) -> Optional[TaskGetOutputResponse]:
        data = {
            "taskId": task_id,
        }

        response = self._request(
            method='POST',
            url='/user-api/v1/task/get-output',
            data=data,
            token=token,
        )

        result = TaskGetOutputResponse.model_validate(response) if response else None
        return result if result else None

    def get_rented_storage_list(
            self,
            token: str,
            statuses: List[RentedStatusEnumDto],
            query: Optional[str] = None,
            page: int = 1,
            limit: int = 10,
    ) -> Tuple[List[StorageRentedDto], int]:

        data = {
            #"statuses": [item.value for item in statuses] if statuses else [],
            "entityFilterRequest": {
                "orderByField": "id",
                "orderByDirection": "DESC",
                "page": page,
                "limit": limit
            },
            "queryString": query
        }

        if statuses:
            data['statuses'] = [item.value for item in statuses] if statuses else []

        response = self._request(
            method='POST',
            url='/user-api/v1/storage-rented/list',
            data=data,
            token=token,
        )

        result = StorageRentedListResponse.model_validate(response) if response else None
        return (result.entities, result.pagination_data.total_pages) if result else ([], None)

    def get_rented_instance_list(
            self,
            token: str,
            statuses: List[str],
            query: Optional[str] = None,
            page: int = 1,
            limit: int = 10,
    ) -> Tuple[List[InstanceRentedDto], int]:
        data = {
            #"statuses": [item.value for item in statuses],
            "entityFilterRequest": {
                "orderByField": "id",
                "orderByDirection": "DESC",
                "page": page,
                "limit": limit
            },
            "queryString": query
        }

        if statuses:
            data['businessStatuses'] = statuses

        response = self._request(
            method='POST',
            url='/user-api/v1/instance-rented/list',
            data=data,
            token=token,
        )

        result = InstanceRentedListResponse.model_validate(response) if response else None
        return (result.paginated_list.entities,
                result.paginated_list.pagination_data.total_pages) if result and result.paginated_list else ([], None)

    def get_rented_business_status_map(self, token: str, ) -> Optional[Dict[str, str]]:
        response = self._request(
            method='POST',
            url='/user-api/v1/instance-rented/business-status-localized-map',
            data=None,
            token=token,
        )

        data = InstanceRentedBusinessStatusMapperResponse.model_validate(response) if response else None

        return data.instance_rented_business_status_map if data else None

    def get_rented_item(
            self,
            token: str,
            instance_slug: Optional[str] = None,
            instance_id: Optional[int] = None,
    ) -> Optional[InstanceRentedDto]:
        if not instance_slug and not instance_id:
            return None

        data = {
            "instanceRentedSlug": instance_slug,
        }

        response = self._request(
            method='POST',
            url='/user-api/v1/instance-rented/view-by-slug',
            data=data,
            token=token,
        )

        return InstanceRentedItemResponse.model_validate(response).instance_rented if response else None

    def get_selfhosted_item(
            self,
            token: str,
            instance_slug: Optional[str] = None,
            instance_id: Optional[int] = None,
    ) -> Optional[SelfHostedInstanceDto]:
        if not instance_slug and not instance_id:
            return None

        data = {
            "selfhostedInstanceSlug": instance_slug,
        }

        response = self._request(
            method='POST',
            url='/user-api/v1/self-hosted-instance/view-by-slug',
            data=data,
            token=token,
        )

        return SelfHostedRentedItemResponse.model_validate(response).selfhosted_instance if response else None

    def get_selfhosted_instance_list(
            self,
            token: str,
            statuses: List[str],
            query: Optional[str] = None,
            page: int = 1,
            limit: int = 10,
    ) -> Tuple[List[SelfHostedInstanceDto], int]:
        data = {
            #"statuses": [item.value for item in statuses] if statuses else [],
            "entityFilterRequest": {
                "orderByField": "id",
                "orderByDirection": "DESC",
                "page": page,
                "limit": limit
            },
            "queryString": query
        }

        if statuses:
            data['businessStatuses'] = statuses

        response = self._request(
            method='POST',
            url='/user-api/v1/self-hosted-instance/list',
            data=data,
            token=token,
        )

        result = SelfHostedInstanceListResponse.model_validate(response) if response else None
        return (result.paginated_list.entities,
                result.paginated_list.pagination_data.total_pages) if result and result.paginated_list else ([], None)

    def get_selfhosted_business_status_map(self, token: str, ) -> Optional[Dict[str, str]]:
        response = self._request(
            method='POST',
            url='/user-api/v1/self-hosted-instance/business-status-localized-map',
            data=None,
            token=token,
        )

        data = SelfHostedRentedRentedBusinessStatusMapperResponse.model_validate(response) if response else None
        return data.selfhosted_instance_business_status_map if data else None

    def get_profile(
            self,
            token: str,
    ) -> Optional[UserProfileResponse]:

        response = self._request(
            method='GET',
            url='/frontend-api/user/my-profile',
            token=token,
        )

        result = UserProfileResponse.model_validate(response) if response else None
        return result if result else None

    def stop_task_on_instance(
            self,
            token: str,
            task_id: int,
    ) -> Optional[TheStageBaseResponse]:
        data = {
            "taskId": task_id,
        }

        response = self._request(
            method='POST',
            url='/user-api/v1/task/cancel-task',
            data=data,
            token=token,
        )

        result = TheStageBaseResponse.model_validate(response) if response else None
        return result if result else None

    def get_container_list(
            self,
            token: str,
            query_string: Optional[str] = None,
            instance_rented_id: Optional[int] = None,
            selfhosted_instance_id: Optional[int] = None,
            project_id: Optional[int] = None,
            statuses: List[str] = [],
            page: int = 1,
            limit: int = 10,
    ) -> Tuple[List[DockerContainerDto], int]:
        data = {
            "entityFilterRequest": {
                "orderByField": "id",
                "orderByDirection": "DESC",
                "page": page,
                "limit": limit
            },
            "instanceRentedId": instance_rented_id,
            "selfhostedInstanceId": selfhosted_instance_id,
            "projectId": project_id,
            "queryString": query_string,
            "statuses": statuses
        }

        response = self._request(
            method='POST',
            url='/user-api/v1/docker-container/list',
            data=data,
            token=token,
        )

        result = DockerContainerListResponse.model_validate(response) if response else None
        return (result.paginated_list.entities,
                result.paginated_list.pagination_data.total_pages) if result and result.is_success and result.paginated_list else (
        [], None)

    def get_container_item(
            self,
            token: str,
            container_slug: Optional[str] = None,
            container_id: Optional[int] = None,
    ) -> Optional[DockerContainerItemResponse]:
        if not container_slug and not container_id:
            return None

        if container_id:
            data = {
                "dockerContainerId": container_id,
            }
        else:
            data = {
                "dockerContainerSlug": container_slug,
            }

        response = self._request(
            method='POST',
            url='/user-api/v1/docker-container/view',
            data=data,
            token=token,
        )

        return DockerContainerItemResponse.model_validate(response).docker_container if response else None

    def container_action(
            self,
            token: str,
            request_param: DockerContainerActionRequestDto,
    ) -> bool:

        response = self._request(
            method='POST',
            url='/user-api/v1/docker-container/action',
            data=request_param.model_dump(by_alias=True),
            token=token,
        )

        result = TheStageBasePaginatedResponse.model_validate(response) if response else None
        return result.is_success if result and result.is_success else False

    def get_container_business_status_map(self, token: str, ) -> Optional[Dict[str, str]]:
        response = self._request(
            method='POST',
            url='/user-api/v1/docker-container/status-localized-mapping',
            data=None,
            token=token,
        )

        data = ContainerBusinessStatusMapperResponse.model_validate(response) if response else None
        return data.docker_container_status_map if data else None

    def get_project_by_slug(self, slug: str, token: str) -> Optional[ProjectDto]:
        data = {
            "projectSlug": slug,
        }

        response = self._request(
            method='POST',
            url='/user-api/v1/project/view-by-slug',
            data=data,
            token=token,
        )

        result = ProjectViewResponse.model_validate(response) if response else None
        project = ProjectDto.model_validate(result.project) if result else None
        return project if result and result.is_success else None

    def get_project_deploy_ssh_key(self, slug: str, token: str) -> str:
        request = ProjectGetDeploySshKeyRequest(
            projectSlug=slug
        )

        response = self._request(
            method='POST',
            url='/user-api/v1/project/get-deploy-ssh-key',
            data=request.model_dump(),
            token=token,
        )

        result = ProjectGetDeploySshKeyResponse.model_validate(response) if response else None
        return result.privateKey if result and result.is_success else None

    def execute_project_task(
            self,
            token: str,
            project_slug: str,
            run_command: str,
            task_title: str,
            docker_container_slug: Optional[str] = None,
            commit_hash: Optional[str] = None,
            task_description: Optional[str] = None,
    ) -> Optional[ProjectRunTaskResponse]:
        request = ProjectRunTaskRequest(
            projectSlug=project_slug,
            dockerContainerSlug=docker_container_slug,
            commitHash=commit_hash,
            runCommand=run_command,
            taskTitle=task_title,
            taskDescription=task_description,
        )

        response = self._request(
            method='POST',
            url='/user-api/v1/project/execute-task',
            data=request.model_dump(),
            token=token,
        )

        return ProjectRunTaskResponse.model_validate(response) if response else None



from typing import Optional

from pydantic import BaseModel, ConfigDict, Field


class StorageEntity(BaseModel):
    model_config = ConfigDict(use_enum_values=True)

    slug: Optional[str] = Field(None, alias='slug')
    title: Optional[str] = Field(None, alias='title')
    provider_id: Optional[str] = Field(None, alias='provider_id')
    status: Optional[str] = Field(None, alias='status')
    is_active: Optional[bool] = Field(False, alias='is_active')
    created_at: Optional[str] = Field(None, alias='created_at')
    updated_at: Optional[str] = Field(None, alias='updated_at')



from pydantic import Field, ConfigDict, BaseModel

from thestage.services.clients.thestage_api.dtos.base_response import TheStageBaseResponse


class ProjectGetDeploySshKeyResponse(TheStageBaseResponse):
    model_config = ConfigDict(use_enum_values=True)

    publicKey: str = Field(None, alias='publicKey')
    privateKey: str = Field(None, alias='privateKey')



from enum import Enum
from typing import List


class DaemonStatusEnumDto(str, Enum):
    NEW: str = 'NEW'
    ONLINE: str = 'ONLINE'
    OFFLINE: str = 'OFFLINE'
    UNKNOWN: str = 'UNKNOWN'
    ALL: str = 'ALL'

    @staticmethod
    def find_special_status(statuses: List['DaemonStatusEnumDto']) -> bool:
        q = list(filter(lambda x: True if x == DaemonStatusEnumDto.ALL else False, statuses))
        return True if q else False



from typing import Optional, List

from pydantic import BaseModel, Field
from thestage_core.entities.file_item import FileItemEntity


class SftpFileItemEntity(FileItemEntity):

    instance_path: Optional[str] = Field(None)
    container_path: Optional[str] = Field(None)
    tmp_path: Optional[str] = Field(None)
    children: List['SftpFileItemEntity'] = Field(default=[])
    dest_path: Optional[str] = Field(None)



class SftpPathHelper(BaseModel):

    tmp_container_path: str = Field(None)
    tmp_instance_path: str = Field(None)

    tmp_folder_path: str = Field(None)

    item_name: str = Field(None)

    item_path: str = Field(None)
    item_dop_path: str = Field(None)


    def tmp_container_path_with_name(self) -> str:
        return f"{self.tmp_container_path}/{self.item_name}"

    def tmp_instance_path_with_name(self) -> str:
        return f"{self.tmp_instance_path}/{self.item_name}"



import logging
import platform
from logging.handlers import RotatingFileHandler
from pathlib import Path
from typing import Any

import typer
from thestage_core.config import THESTAGE_CONFIG_DIR, THESTAGE_CONFIG_FILE
from thestage_core.exceptions.file_system_exception import FileSystemException

from thestage.config import THESTAGE_LOGGING_FILE


def get_log_path_from_os() -> Path:
    system = platform.system()
    if system == 'Linux':
        path = Path.home().joinpath(THESTAGE_CONFIG_DIR).joinpath('logs')
    elif system == 'Windows':
        path = Path.home().joinpath(THESTAGE_CONFIG_DIR).joinpath('logs')
    elif system == 'Darwin':
        path = Path.home().joinpath('library').joinpath('logs').joinpath('thestage')
    else:
        path = Path.home().joinpath(THESTAGE_CONFIG_DIR).joinpath('logs')

    if not path.exists():
        try:
            path.mkdir(exist_ok=True, parents=True)
        except OSError:
            raise FileSystemException("Error create log dir")

    return path


def build_logger(level) -> logging.Logger:
    logger = logging.getLogger("thestage_log")
    formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    logger.setLevel(level)
    # logger.remove()

    log_path = get_log_path_from_os()
    tsr_log_file = log_path.joinpath(THESTAGE_LOGGING_FILE)

    if tsr_log_file:
        file_h = RotatingFileHandler(filename=tsr_log_file, maxBytes=1024, backupCount=5)
        file_h.setLevel(level)
        file_h.setFormatter(formatter)
        logger.addHandler(file_h)

    return logger


app_logger = build_logger(level=logging.DEBUG)



from typing import Optional, Annotated

from tabulate import tabulate
from thestage_core.entities.config_entity import ConfigEntity

from thestage.entities.project_task import ProjectTaskEntity
from thestage.helpers.error_handler import error_handler
from thestage.i18n.translation import __
from thestage.services.clients.thestage_api.dtos.task_response import TaskDto
from thestage.helpers.logger.app_logger import app_logger
from thestage.controllers.utils_controller import validate_config_and_get_service_factory, get_current_directory

import typer

from thestage.services.project.dto.project_config import ProjectConfig
from thestage.services.project.project_service import ProjectService

app = typer.Typer(no_args_is_help=True, help=__("Help working with projects"))

@app.command(name='clone')
def clone(
        project_slug: str = typer.Option(
            None,
            '--project-uid',
            '-uid',
            help=__("Slug for project (Required)"),
            is_eager=False,
        ),
        working_directory: Optional[str] = typer.Option(
            None,
            "--working-directory",
            "-wd",
            help=__("Full path for working directory"),
            is_eager=False,
        ),
        no_dialog: Optional[bool] = typer.Option(
            None,
            "--no-dialog",
            "-nd",
            help=__("Start process with default values, without future dialog"),
            is_eager=False,
        ),
):
    """
        Clone project on current working directory
    """
    path = get_current_directory()
    app_logger.info(f'Start project clone from {get_current_directory()}')

    service_factory = validate_config_and_get_service_factory(no_dialog=no_dialog, working_directory=working_directory)
    config = service_factory.get_config_provider().get_full_config()

    project_service = service_factory.get_project_service()
    # file_system_service = service_factory.get_file_system_service()

    project_service.clone_project(
        config=config,
        project_slug=project_slug,
        no_dialog=no_dialog,
    )

    typer.echo(__("Clone done"))
    raise typer.Exit(0)


@app.command(name='init')
def init(
        project_slug: Optional[str] = typer.Option(
            None,
            '--project-uid',
            '-uid',
            help=__("Project Unique ID"),
            is_eager=False,
        ),
        working_directory: Optional[str] = typer.Option(
            None,
            "--working-directory",
            "-wd",
            help=__("Full path for working directory"),
            is_eager=False,
        ),
        no_dialog: Optional[bool] = typer.Option(
            None,
            "--no-dialog",
            "-nd",
            help=__("Start process with default values, without future dialog"),
            is_eager=False,
        ),
):
    """
        Initialize project on current working directory
    """
    app_logger.info(f'Start project init from {get_current_directory()}')

    service_factory = validate_config_and_get_service_factory(no_dialog=no_dialog, working_directory=working_directory)
    config = service_factory.get_config_provider().get_full_config()

    project_service = service_factory.get_project_service()
    project_config = service_factory.get_config_provider().read_project_config()

    if project_config:
        typer.echo(__("Folder initialized, it contains a working project"))
        raise typer.Exit(1)

    project_service.init_project(
        project_slug=project_slug,
        config=config,
        no_dialog=no_dialog,
    )

    typer.echo(__("Init done"))
    raise typer.Exit(0)


@app.command(name='run')
def run(
        command: str = typer.Option(
            None,
            '--command',
            '-com',
            help=__("Command for run (Required)"),
            is_eager=True,
        ),
        commit_hash: Optional[str] = typer.Option(
            None,
            '--commit-hash',
            '-hash',
            help=__("Commit hash"),
            is_eager=False,
        ),
        task_title: Optional[str] = typer.Option(
            None,
            '--task-title',
            '-t',
            help=__("Title for task"),
            is_eager=False,
        ),
        task_description: Optional[str] = typer.Option(
            None,
            '--task-description',
            '-d',
            help=__("Description for task"),
            is_eager=False,
        ),
        docker_container_slug: Optional[str] = typer.Option(
            None,
            '--docker-container-uid',
            '-dcuid',
            help=__("Docker Container Unique ID"),
            is_eager=False,
        ),
        working_directory: Optional[str] = typer.Option(
            None,
            "--working-directory",
            "-wd",
            help=__("Full path for working directory"),
            show_default=False,
            is_eager=False,
        ),
        no_dialog: Optional[bool] = typer.Option(
            None,
            "--no-dialog",
            "-nd",
            help=__("Start process with default values, without future dialog"),
            is_eager=False,
        ),
        auto_commit: Optional[bool] = typer.Option(
            False,
            "--auto-commit",
            "-ac",
            help=__("Use with no_dialog config, default disable auto commit"),
            is_eager=False,
        ),
):
    """
        Execute a task from a project
    """
    app_logger.info(f'Start project run from {get_current_directory()}')

    if not command:
        typer.echo(__('Command is required'))
        raise typer.Exit(1)

    if not docker_container_slug:
        typer.echo(__('Docker container Unique ID is required'))
        raise typer.Exit(1)

    service_factory = validate_config_and_get_service_factory(no_dialog=no_dialog, working_directory=working_directory)
    config = service_factory.get_config_provider().get_full_config()

    project_service = service_factory.get_project_service()
    project_config: ProjectConfig = service_factory.get_config_provider().read_project_config()

    if not project_config:
        typer.echo(__("This folder is not initialize, please doing init or clone"))
        raise typer.Exit(1)

    task: Optional[TaskDto] = project_service.project_run_task(
        config=config,
        project_config=project_config,
        run_command=command,
        commit_hash=commit_hash,
        task_title=task_title,
        task_description=task_description,
        docker_container_slug=docker_container_slug,
        no_dialog=no_dialog,
        auto_commit=auto_commit,
    )
    if task:
        typer.echo(__(
            "Task id: %task_id% - %task_status%",
            {
                'task_id': str(task.id),
                'task_status': task.frontend_status.status_translation
            }
        ))

    typer.echo(__("Task successfully started"))
    raise typer.Exit(0)


@app.command("task-ls")
def list_runs(
        working_directory: Optional[str] = typer.Option(
            None,
            "--working-directory",
            "-wd",
            help=__("Full path for working directory"),
            is_eager=False,
        ),
        row: int = typer.Option(
            5,
            '--row',
            '-r',
            help=__("Count row in table"),
            is_eager=False,
        ),
        page: int = typer.Option(
            1,
            '--page',
            '-p',
            help=__("Page number"),
            is_eager=False,
        ),
        no_dialog: Optional[bool] = typer.Option(
            None,
            "--no-dialog",
            "-nd",
            help=__("Start process with default values, without future dialog"),
            is_eager=False,
        ),
):
    """
        List tasks started from project
    """
    app_logger.info(f'Start project list-runs from {get_current_directory()}')

    service_factory = validate_config_and_get_service_factory(no_dialog=no_dialog, working_directory=working_directory)
    config = service_factory.get_config_provider().get_full_config()

    project_service: ProjectService = service_factory.get_project_service()

    project_config: ProjectConfig = service_factory.get_config_provider().read_project_config()
    if not project_config:
        typer.echo(__("This folder is not initialize, please doing init or clone"))
        raise typer.Exit(1)

    print_project_task_list(
        config=config,
        project_slug=project_config.slug,
        project_service=project_service,
        row=row,
        page=page,
        no_dialog=no_dialog,
    )
    typer.echo(__("List runs done"))
    raise typer.Exit(0)


def print_project_task_list(
        config: ConfigEntity,
        project_slug: str,
        project_service: ProjectService,
        row: int = 5,
        page: int = 1,
        no_dialog: bool = False,
):
    data, pagination_data = project_service.get_project_task_list(
        project_slug=project_slug,
        config=config,
        row=row,
        page=page,
    )

    raw_data = [list(item.model_dump(by_alias=True).values()) for item in data]

    typer.echo(f"Page: {page}, Limit: {row}, Total Pages: {pagination_data.total_pages} ")
    raw_data.insert(0, ['#'] + list(map(lambda x: x.alias, ProjectTaskEntity.model_fields.values())))
    typer.echo(tabulate(raw_data, showindex="always", tablefmt="double_outline", headers="firstrow"))

    if len(raw_data) == 1:
        typer.echo("List empty, work done")
        raise typer.Exit(0)

    if no_dialog:
        raise typer.Exit(0)

    if page == pagination_data.total_pages:
        typer.echo(__("No more items to show"))
        raise typer.Exit(0)

    next_page: int = typer.prompt(
        text=f'Go to next page (0 to exit)?',
        default=page + 1,
        show_choices=False,
        type=int,
        show_default=True,
    )
    if next_page == 0:
        raise typer.Exit(0)
    else:
        print_project_task_list(
            config=config,
            project_slug=project_slug,
            project_service=project_service,
            row=row,
            page=next_page,
        )



from typing import Optional, Tuple, List

import click
import typer
from git import Commit
from thestage_core.entities.config_entity import ConfigEntity
from thestage_core.entities.enums.commit_question_response import CommitQuestionResponse
from thestage_core.services.filesystem_service import FileSystemServiceCore

from thestage.entities.enums.yes_no_response import YesOrNoResponse
from thestage.entities.project_task import ProjectTaskEntity
from thestage.exceptions.git_access_exception import GitAccessException
from thestage.i18n.translation import __
from thestage.services.clients.git.git_client import GitLocalClient
from thestage.services.clients.thestage_api.dtos.pagination_data import PaginationData
from thestage.services.clients.thestage_api.dtos.project_controller.project_run_task_response import \
    ProjectRunTaskResponse
from thestage.services.clients.thestage_api.dtos.project_response import ProjectDto
from thestage.services.clients.thestage_api.dtos.task_controller.task_list_for_project_response import \
    TaskListForProjectPaging
from thestage.services.clients.thestage_api.dtos.task_response import TaskDto
from thestage.services.project.dto.project_config import ProjectConfig
from thestage.services.project.mapper.project_task_mapper import ProjectTaskMapper
from thestage.services.remote_server_service import RemoteServerService
from thestage.services.abstract_service import AbstractService
from thestage.helpers.error_handler import error_handler
from thestage.services.clients.thestage_api.api_client import TheStageApiClient
from thestage.services.config_provider.config_provider import ConfigProvider


class ProjectService(AbstractService):
    __thestage_api_client: TheStageApiClient = None

    def __init__(
            self,
            thestage_api_client: TheStageApiClient,
            config_provider: ConfigProvider,
            remote_server_service: RemoteServerService,
            file_system_service: FileSystemServiceCore,
            git_local_client: GitLocalClient,
    ):
        super(ProjectService, self).__init__(
            config_provider=config_provider
        )
        self.__thestage_api_client = thestage_api_client
        self.__remote_server_service = remote_server_service
        self.__file_system_service = file_system_service
        self.__git_local_client = git_local_client
        self.__project_task_mapper = ProjectTaskMapper()
        self.__config_provider = config_provider

    @error_handler()
    def get_project_by_slug_and_check_if_present(
            self,
            config: ConfigEntity,
            project_slug: Optional[str] = None,
            no_dialog: bool = False,
    ) -> Optional[ProjectDto]:
        if not project_slug:

            if no_dialog:
                typer.echo(__('In no dialog mode, you need added project slug as params'))
                raise typer.Exit(1)

            typer.echo(__('Please go to site and create new project or select present'))
            project_slug: str = typer.prompt(
                text=__('Please give me sketch slug'),
                show_choices=False,
                type=str,
                show_default=False,
            )
            if not project_slug:
                typer.echo(__('Slug can not be empty'))
                raise typer.Exit(1)

        project: Optional[ProjectDto] = self.__thestage_api_client.get_project_by_slug(
            slug=project_slug,
            token=config.main.auth_token,
        )

        if not project:
            typer.echo(__('You entered the wrong slug, sketch not found'))
            raise typer.Exit(1)

        return project

    @error_handler()
    def init_project(
            self,
            config: ConfigEntity,
            project_slug: str,
            no_dialog: bool
    ):
        project = self.get_project_by_slug_and_check_if_present(
            config=config,
            project_slug=project_slug,
            no_dialog=no_dialog,
        )

        is_git_folder = self.__git_local_client.is_present_local_git(
            path=config.runtime.working_directory,
        )
        if is_git_folder:
            has_remote = self.__git_local_client.has_remote(
                path=config.runtime.working_directory,
            )
            if has_remote:
                typer.echo(__('You have local repo with remote, we can not work with this'))
                raise typer.Exit(1)

        if not project.git_repository_url:
            typer.echo(__('Sketch dont have git repository url'))
            raise typer.Exit(1)

        if project.last_commit_hash or project.last_commit_description:
            prompt_choices = click.Choice(['YES', 'NO'])
            continue_with_non_empty_repo: YesOrNoResponse = typer.prompt(
                text=__('Remote repository is probably not empty: latest commit is "{commit_description}" (sha: {commit_hash})\nDo you wish to continue?').format(commit_description=project.last_commit_description, commit_hash=project.last_commit_hash),
                show_choices=True,
                default=YesOrNoResponse.YES.value,
                type=prompt_choices,
                show_default=True,
            )
            if continue_with_non_empty_repo == "NO":
                typer.echo(__('Project init aborted'))
                raise typer.Exit(0)

        deploy_ssh_key = self.__thestage_api_client.get_project_deploy_ssh_key(
            slug=project.slug,
            token=config.main.auth_token
        )

        deploy_key_path = self.__config_provider.save_project_deploy_ssh_key(
            deploy_ssh_key=deploy_ssh_key,
            slug=project.slug
        )

        if is_git_folder:
            has_changes = self.__git_local_client.has_changes(
                path=config.runtime.working_directory,
            )
            if has_changes:
                typer.echo(__('You local repo has changes and not empty, please create empty folder'))
                raise typer.Exit(1)
        else:
            repo = self.__git_local_client.init_repository(
                path=config.runtime.working_directory,
            )

        is_remote_added = self.__git_local_client.add_remote_to_repo(
            path=config.runtime.working_directory,
            remote_url=project.git_repository_url,
            remote_name=project.git_repository_name,
        )
        if not is_remote_added:
            typer.echo(__('We can not add remote, something wrong'))
            raise typer.Exit(2)

        self.__git_local_client.git_fetch(path=config.runtime.working_directory, deploy_key_path=deploy_key_path)

        branch = self.__git_local_client.find_main_branch_name(path=config.runtime.working_directory, )
        if branch:
            self.__git_local_client.git_pull(path=config.runtime.working_directory, deploy_key_path=deploy_key_path,
                                             branch=branch)

        self.__git_local_client.init_gitignore(path=config.runtime.working_directory)

        self.__git_local_client.git_add_all(repo_path=config.runtime.working_directory)

        project_config = ProjectConfig()
        project_config.id = project.id
        project_config.slug = project.slug
        project_config.git_repository_url = project.git_repository_url
        project_config.deploy_key_path = str(deploy_key_path)
        self.__config_provider.save_project_config(project_config=project_config)

    @error_handler()
    def clone_project(
            self,
            config: ConfigEntity,
            project_slug: str,
            no_dialog: bool
    ):
        if not self.__file_system_service.is_folder_empty(folder=config.runtime.working_directory, auto_create=True):
            typer.echo(__("This folder is not empty, we can not clone here"))
            raise typer.Exit(1)

        project = self.get_project_by_slug_and_check_if_present(
            config=config,
            project_slug=project_slug,
            no_dialog=no_dialog,
        )

        is_git_folder = self.__git_local_client.is_present_local_git(
            path=config.runtime.working_directory,
        )

        if is_git_folder:
            typer.echo(__('You have local repo, we can not work with this'))
            raise typer.Exit(1)

        if not project.git_repository_url:
            typer.echo(__("Unexpected Project error, missing Repository"))
            raise typer.Exit(1)

        deploy_ssh_key = self.__thestage_api_client.get_project_deploy_ssh_key(slug=project.slug,
                                                                               token=config.main.auth_token)
        deploy_key_path = self.__config_provider.save_project_deploy_ssh_key(deploy_ssh_key=deploy_ssh_key,
                                                                             slug=project.slug)

        try:
            self.__git_local_client.clone(
                url=project.git_repository_url,
                path=config.runtime.working_directory,
                deploy_key_path=deploy_key_path
            )
            self.__git_local_client.init_gitignore(path=config.runtime.working_directory)
        except GitAccessException as ex:
            typer.echo(ex.get_message())
            typer.echo(ex.get_dop_message())
            typer.echo(__(
                "Please check you mail or open this repo url %git_url% and 'Accept invitation'",
                {
                    'git_url': ex.get_url()
                }
            ))
            typer.Exit(1)

        project_config = ProjectConfig()
        project_config.id = project.id
        project_config.slug = project.slug
        project_config.git_repository_url = project.git_repository_url
        project_config.deploy_key_path = str(deploy_key_path)
        self.__config_provider.save_project_config(project_config=project_config)

    @error_handler()
    def project_run_task(
            self,
            config: ConfigEntity,
            project_config: ProjectConfig,
            run_command: str,
            task_title: Optional[str] = None,
            commit_hash: Optional[str] = None,
            task_description: Optional[str] = None,
            docker_container_slug: Optional[str] = None,
            no_dialog: bool = False,
            auto_commit: bool = False,
    ) -> Optional[TaskDto]:
        if not commit_hash:
            is_git_folder = self.__git_local_client.is_present_local_git(path=config.runtime.working_directory)
            if not is_git_folder:
                typer.echo(__("This folder dont have git project"))
                raise typer.Exit(1)

            has_changes = self.__git_local_client.has_changes(
                path=config.runtime.working_directory,
            )
            if has_changes:
                typer.echo(__('You have not commit changes'))

                commit_name: Optional[str] = None

                if not no_dialog:
                    branch_name = self.__git_local_client.get_active_branch_name(config.runtime.working_directory)

                    response: CommitQuestionResponse = typer.prompt(
                        text=__('Active branch [{branch_name}] has uncommitted changes. COMMIT / EXIT?').format(
                            branch_name=branch_name),
                        show_choices=True,
                        default=CommitQuestionResponse.COMMIT.value,
                        type=CommitQuestionResponse,
                        show_default=True,
                    )
                    if response == CommitQuestionResponse.EXIT:
                        raise typer.Exit(0)

                    commit_name = typer.prompt(
                        text=__('Please give me commit name'),
                        show_choices=False,
                        type=str,
                        show_default=False,
                    )
                else:
                    if auto_commit:
                        commit_name = __("Auto created name for commit by tsr")

                if commit_name:
                    task_title = task_title if task_title else commit_name
                    commit_result = self.__git_local_client.commit_local_changes(
                        path=config.runtime.working_directory,
                        name=commit_name
                    )

                    if commit_result:
                        # in docs not Commit object, on real - str
                        if isinstance(commit_result, str):
                            typer.echo(commit_result)

                        commit = self.__git_local_client.get_latest_commit(path=config.runtime.working_directory)
                        if commit and isinstance(commit, Commit):
                            commit_hash = commit.hexsha

                    self.__git_local_client.push_changes(
                        path=config.runtime.working_directory,
                        deploy_key_path=project_config.deploy_key_path
                    )
                    typer.echo(__("Pushed changes to remote repository"))
                else:
                    if not no_dialog:
                        typer.echo(__('Cannot commit with empty commit name, your code run without last changes.'))

        if not task_title:
            typer.echo(__("Task title is required, you can input manual or auto generated"))

            auto_title = "Task for project {slug}".format(slug=project_config.slug)
            if not no_dialog:
                title: str = typer.prompt(
                    text=__('New task title:'),
                    show_choices=False,
                    default=auto_title,
                    type=str,
                    show_default=True,
                )
                if not title:
                    typer.echo(__("Task title can not be empty"))
                    raise typer.Exit(1)
                else:
                    task_title = title
            else:
                task_title = auto_title

        run_task_response: ProjectRunTaskResponse = self.__thestage_api_client.execute_project_task(
            token=config.main.auth_token,
            project_slug=project_config.slug,
            docker_container_slug=docker_container_slug,
            run_command=run_command,
            commit_hash=commit_hash,
            task_title=task_title,
            task_description=task_description,
        )
        if run_task_response:
            if run_task_response.is_success and run_task_response.task:
                typer.echo(__("Task run on server"))
                return run_task_response.task
            else:
                typer.echo(__(
                    'The task failed with an error: %server_massage%',
                    {'server_massage': run_task_response.message or ""}
                ))
                raise typer.Exit(1)
        else:
            typer.echo(__("The task failed with an error"))
            raise typer.Exit(1)

    @error_handler()
    def get_project_task_list(
            self,
            config: ConfigEntity,
            project_slug: str,
            row: int = 5,
            page: int = 1,
    ) -> Tuple[List[ProjectTaskEntity], PaginationData]:
        data: Optional[TaskListForProjectPaging] = self.__thestage_api_client.get_task_list_for_project(
            token=config.main.auth_token,
            project_slug=project_slug,
            page=page,
            limit=row,
        )

        result: List[ProjectTaskEntity] = list(
            map(lambda x: self.__project_task_mapper.build_task_entity(x), data.entities))

        return result, data.pagination_data



from datetime import datetime
from typing import Optional, List

from pydantic import Field, BaseModel, ConfigDict

from thestage.services.clients.thestage_api.dtos.base_response import TheStageBaseResponse
from thestage.services.clients.thestage_api.dtos.pagination_data import PaginationData
from thestage.services.clients.thestage_api.dtos.enums.access_type import AccessTypeEnumDto
from thestage.services.clients.thestage_api.dtos.enums.drive_type import DriveTypeEnumDto
from thestage.services.clients.thestage_api.dtos.cloud_provider_region import CloudProviderRegionDto
from thestage.services.clients.thestage_api.dtos.enums.rented_status import RentedStatusEnumDto
from thestage.services.clients.thestage_api.dtos.enums.storage_type import StorageTypeEnumDto


class StorageRentedDto(BaseModel):
    model_config = ConfigDict(use_enum_values=True)

    id: Optional[int] = Field(None, alias='id')
    #prices_per_hour: Optional[List[PriceDefinitionDto]] = Field(default_factory=list, alias='pricesPerHour')
    #total_stagi_spent: Optional[PriceDefinitionDto] = Field(None, alias='totalStagiSpent')
    fk_client: Optional[int] = Field(None, alias='fkClient')
    slug: Optional[str] = Field(None, alias='slug')
    title: Optional[str] = Field(None, alias='title')
    fk_cloud_provider_region: Optional[int] = Field(None, alias='fkCloudProviderRegion')
    cloud_provider_region: Optional[CloudProviderRegionDto] = Field(None, alias='cloudProviderRegion')
    storage_type: StorageTypeEnumDto = Field(StorageTypeEnumDto.UNKNOWN, alias='storageType')
    drive_type: DriveTypeEnumDto = Field(DriveTypeEnumDto.UNKNOWN, alias='driveType')
    access_type: AccessTypeEnumDto = Field(AccessTypeEnumDto.UNKNOWN, alias='accessType')
    storage_size_gb: Optional[int] = Field(None, alias='storageSizeGb')
    provider_id: Optional[str] = Field(None, alias='providerId')
    is_active: Optional[bool] = Field(False, alias='isActive')
    status: RentedStatusEnumDto = Field(RentedStatusEnumDto.UNKNOWN, alias='status')
    created_at: Optional[datetime] = Field(None, alias='createdAt')
    updated_at: Optional[datetime] = Field(None, alias='updatedAt')
    aws_access_key: Optional[str] = Field(None, alias='awsAccessKey')
    aws_secret_key: Optional[str] = Field(None, alias='awsSecretKey')
    endpoint: Optional[str] = Field(None, alias='endpoint')


class StorageRentedListResponse(TheStageBaseResponse):
    entities: List[StorageRentedDto] = Field(default_factory=list, alias='entities')
    current_page: Optional[int] = Field(None, alias='currentPage')
    last_page: Optional[bool] = Field(None, alias='lastPage')
    total_pages: Optional[int] = Field(None, alias='totalPages')
    pagination_data: Optional[PaginationData] = Field(None, alias='paginationData')




from thestage.i18n.translation import __
from thestage.helpers.logger.app_logger import app_logger
from thestage.controllers.utils_controller import get_current_directory
from thestage import __app_name__, __version__

import typer

app = typer.Typer(no_args_is_help=True)


@app.command(no_args_is_help=False)
def version():
    """
        return Name and Version app
    """
    app_logger.info(f'Start version from {get_current_directory()}')
    typer.echo(
        __("%app_name% v%version%", {'app_name': __app_name__, 'version': __version__}))
    raise typer.Exit(0)



from abc import ABC
from typing import List, Dict, Any, Optional

import typer
from tabulate import tabulate
from thestage_core.entities.config_entity import ConfigEntity

from thestage.helpers.logger.app_logger import app_logger
from thestage.i18n.translation import __
from thestage.services.config_provider.config_provider import ConfigProvider
from thestage.services.storage.mapper.storage_mapper import AbstractMapper


class AbstractService(ABC):

    def __init__(
            self,
            config_provider: ConfigProvider,
    ):
        self._config_provider = config_provider

    def map_frontend_statuses(self, statuses_mapper: Dict[str, str], frontend: List[str]) -> Optional[List[str]]:

        real_statuses = []

        mapper: Dict[str, List] = {}
        for key, value in statuses_mapper.items():

            map_key = value if ' ' not in value else value.replace(' ', '_')
            if value not in mapper:
                mapper[map_key] = [key]
            else:
                mapper[map_key].append(key)

        for item in frontend:
            if item == 'all':
                return []
            elif item == 'unknown':
                continue
            else:
                if item in mapper:
                    real_statuses.extend(mapper[item])
                else:
                    app_logger.error(f'Not found status: {item}')

        return real_statuses

    def print(
            self,
            func_get_data,
            func_special_params: Dict[str, Any],
            mapper: AbstractMapper,
            config: ConfigEntity,
            headers: List[str],
            row: int = 5,
            page: int = 1,
            no_dialog: bool = False,
            show_index: str = 'always',
            max_col_width: List[int] = None,
    ):
        data, total_pages = func_get_data(
            config=config,
            row=row,
            page=page,
            **func_special_params,
        )

        result = list(map(lambda x: mapper.build_entity(x), data))

        if result:
            raw_data = [list(item.model_dump(
                by_alias=True,
                exclude=mapper.get_exclude_fields(),
            ).values()) for item in result]
        else:
            raw_data = [[] for item in headers]
            show_index = 'never'

        typer.echo(__(
            "Page: %page%, Limit: %limit%, Total page: %total_pages%",
            {
                'page': str(page),
                'limit': str(row),
                'total_pages': str(total_pages),
            }
        ))

        typer.echo(tabulate(
            raw_data,
            headers=headers,
            showindex=show_index,
            tablefmt="double_grid",
            #headers="firstrow",
            maxcolwidths=max_col_width,
        ))

        #if len(raw_data) == 1:
        #    typer.echo(__("List empty, work done"))
        #    raise typer.Exit(0)

        if no_dialog:
            raise typer.Exit(0)

        if page >= total_pages:
            raise typer.Exit(0)

        next_page: int = typer.prompt(
            text=__('Go to next page (0 to exit)?'),
            default=page + 1,
            show_choices=False,
            type=int,
            show_default=True,
        )
        if next_page == 0:
            raise typer.Exit(0)
        else:
            self.print(
                func_get_data=func_get_data,
                func_special_params=func_special_params,
                mapper=mapper,
                config=config,
                headers=headers,
                row=row,
                page=next_page,
                no_dialog=no_dialog,
            )



from typing import List, Optional

from pydantic import Field, ConfigDict, BaseModel

from thestage.services.clients.thestage_api.dtos.base_response import TheStageBaseResponse
from thestage.services.clients.thestage_api.dtos.pagination_data import PaginationData
from thestage.services.clients.thestage_api.dtos.task_response import TaskDto


class TaskListForProjectPaging(BaseModel):
    entities: List[TaskDto] = Field(default_factory=list, alias='entities')
    # current_page: Optional[int] = Field(None, alias='currentPage')
    # last_page: Optional[bool] = Field(None, alias='lastPage')
    # total_pages: Optional[int] = Field(None, alias='totalPages')
    pagination_data: Optional[PaginationData] = Field(None, alias='paginationData')


class TaskListForProjectResponse(TheStageBaseResponse):
    model_config = ConfigDict(use_enum_values=True)

    tasks: TaskListForProjectPaging = Field(None, alias='tasks')



from thestage_core.entities.config_entity import ConfigEntity

from thestage.i18n.translation import __
from thestage.helpers.logger.app_logger import app_logger
from thestage.services.config_provider.config_provider import ConfigProvider
from thestage.services.service_facade import ServiceFactory
from thestage.controllers.utils_controller import get_current_directory, validate_config_and_get_service_factory

import typer

app = typer.Typer(no_args_is_help=True, help=__("Help working with config"))


@app.command(name='get', no_args_is_help=False, help=__("Print all config items"))
def config_get():
    """
        Print all use configs
    """
    app_logger.info(f'Start config from {get_current_directory()}')

    service_factory = validate_config_and_get_service_factory(no_dialog=True)
    config = service_factory.get_config_provider().get_full_config()

    if not config:
        typer.echo(__('You dont have config yet'))
        raise typer.Exit(1)

    typer.echo(__('THESTAGE TOKEN: %token%', {'token': config.main.auth_token or ''}))
    typer.echo(__('THESTAGE CONFIG FILE: %dir%', {'dir': config.main.config_file_name or ''}))
    typer.echo(__('THESTAGE API LINK: %link%', {'link': config.main.config_api_link or ''}))
    if config.main.config_local_path:
        typer.echo(__('THESTAGE LOCAL CONFIG PATH: %path%', {'path': str(config.main.config_local_path or '')}))
    if config.main.config_global_path:
        typer.echo(__('THESTAGE GLOBAL CONFIG PATH: %path%', {'path': str(config.main.config_global_path or '')}))

    raise typer.Exit(0)


@app.command(name='set', no_args_is_help=True, help=__("Set some config item"))
def config_set(
    token: str = typer.Option(
            None,
            "--api-token",
            "-t",
            help=__("If you want initialize or change token"),
            is_eager=False,
            is_flag=True,
        ),
):
    """
        Set up some params
    """
    app_logger.info(f'Start config from {get_current_directory()}')

    config_provider = ConfigProvider(local_path=get_current_directory())
    service_factory = ServiceFactory(config_provider)

    app_service = service_factory.get_app_config_service()

    if token:
        app_service.app_change_token(config=config_provider.get_full_config(), token=token)

    typer.echo('Config changed')
    raise typer.Exit(0)


@app.command(name='clear', no_args_is_help=False, help=__("Clear thestage config"))
def config_clear():
    """
        Clear config
    """
    app_logger.info(f'Start config from {get_current_directory()}')
    service_factory = validate_config_and_get_service_factory(no_dialog=False)
    service_factory.get_config_provider().remove_all_config()

    raise typer.Exit(0)



# Init project settings by ENV environment variable
import locale
from pathlib import Path

import typer
import dotenv
from python_gettext_translations.translations import init_translations

from thestage.config.env_base import *

path_env = Path(__file__).parent.parent.resolve()

env_file = path_env.joinpath('.env')

if not env_file.exists():
    path_env = Path(__file__).parent.parent.parent.resolve()
    env_file = path_env.joinpath('.env')

config = None
if env_file:
    # пока не будет делать, затирает все что делаем
    #dotenv.load_dotenv(env_file, override=True)
    config = dotenv.dotenv_values(env_file)

THESTAGE_LOCAL_LANGUAGE = 'en_GB'
if locale.getlocale():
    THESTAGE_LOCAL_LANGUAGE = locale.getlocale()[0]

translation = Path(f'i18n/')

if translation.exists() and translation.is_dir():
    init_translations(f'i18n/')

ENV: str = config.get("THESTAGE_CLI_ENV", "DEV") if config else 'DEV'

if ENV == "PROD":
    from thestage.config.env_prod import *
elif ENV == "STAGE":
    from thestage.config.env_stage import *
else:
    from thestage.config.env_dev import *



from typing import List, Tuple, Optional

from thestage_core.entities.config_entity import ConfigEntity

from thestage.services.clients.thestage_api.dtos.enums.rented_status import RentedStatusEnumDto
from thestage.services.clients.thestage_api.dtos.storage_rented_response import StorageRentedDto
from thestage.services.abstract_service import AbstractService
from thestage.helpers.error_handler import error_handler
from thestage.services.clients.thestage_api.api_client import TheStageApiClient
from thestage.services.config_provider.config_provider import ConfigProvider


class StorageService(AbstractService):

    __thestage_api_client: TheStageApiClient = None

    def __init__(
            self,
            thestage_api_client: TheStageApiClient,
            config_provider: ConfigProvider,
    ):
        super(StorageService, self).__init__(
            config_provider=config_provider
        )

        self.__thestage_api_client = thestage_api_client

    @error_handler()
    def get_list(
            self,
            config: ConfigEntity,
            statuses: List[RentedStatusEnumDto],
            row: int = 5,
            page: int = 1,
    ) -> Tuple[List[StorageRentedDto], int]:

        data, total_pages = self.__thestage_api_client.get_rented_storage_list(
            token=config.main.auth_token,
            statuses=statuses,
            page=page,
            limit=row,
        )

        return data, total_pages



from abc import ABC
from typing import Optional, Any

from thestage.services.abstract_mapper import AbstractMapper
from thestage.entities.storage import StorageEntity
from thestage.services.clients.thestage_api.dtos.storage_rented_response import StorageRentedDto


class StorageMapper(AbstractMapper):

    @staticmethod
    def build_entity(item: StorageRentedDto) -> Optional[StorageEntity]:
        if not item:
            return None

        return StorageEntity(
            slug=item.slug,
            title=item.title,
            provider_id=item.provider_id,
            status=item.status,
            is_active=item.is_active,
            created_at=item.created_at.strftime("%Y-%m-%d %H:%M:%S") if item.created_at else '',
            updated_at=item.updated_at.strftime("%Y-%m-%d %H:%M:%S") if item.updated_at else '',
        )



from enum import Enum
from typing import List


class SelfHostedBusinessStatusEnumDto(str, Enum):

    AWAITING_CONFIGURATION: str = 'AWAITING_CONFIGURATION'
    RUNNING: str = 'RUNNING'
    TERMINATED: str = 'TERMINATED'
    DELETED: str = 'DELETED'
    UNKNOWN: str = 'UNKNOWN'
    ALL: str = 'ALL'

    @staticmethod
    def find_special_status(statuses: List['SelfHostedBusinessStatusEnumDto']) -> bool:
        q = list(filter(lambda x: True if x == SelfHostedBusinessStatusEnumDto.ALL else False, statuses))
        return True if q else False


class SelfHostedFrontendStatusEnumDto(str, Enum):
    terminated: str = 'terminated'
    awaiting_setup: str = 'awaiting_setup'
    online: str = 'online'
    deleted: str = 'deleted'
    unknown: str = 'unknown'
    all: str = 'all'

    @staticmethod
    def find_special_status(statuses: List['SelfHostedFrontendStatusEnumDto']) -> bool:
        q = list(filter(lambda x: True if x == SelfHostedFrontendStatusEnumDto.all else False, statuses))
        return True if q else False
